<?php
/*
 * Copyright (C) 2015, M2X BV
 *
 * Authors: Jean-Paul Saman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/**
 *  @package backup
 *  @subpackage modules
 *  @version $Revision: 2.0 $
 */

if (!defined('W2P_BASE_DIR')) {
    die('You should not access this file directly.');
}

require_once "$baseDir/lib/adodb/adodb-xmlschema.inc.php";
include_once $baseDir . '/modules/backup/zip.lib.php';

/**
 * The Backup class
 */

class CBackup extends w2p_Core_BaseObject {

	public function __construct()
	{
		parent::__construct('backup', 'backup_id');
	}

	public function isValid()
	{
	        $baseErrorMsg = get_class($this) . '::store-check failed -';
	        if ($this->backup_id == 0) {
        	    $this->_error['backup_id'] = $baseErrorMsg . 'backup id not set';
	        }
        	return (count($this->_error)) ? false : true;
	}

	/*
    	public function load($notUsed = null, $backup_id)
	{
        	$q = $this->_getQuery();
	        $q->addQuery('*');
	        $q->addTable('backup');
	        $q->addWhere('backup_id =' . (int)$backup_id);
        	$q->loadObject($this, true, false);
		return true;
    	}
	*/

	public function backup($output_format, $drop_table, $export_what)
	{
		if ($output_format == 'xml') {
  			$schema = new adoSchema($GLOBALS['db']);
			$output = $schema->ExtractSchema(($export_what == 'table') ? false : true);
		} else {
		  	// Build the SQL manually.
			$db->setFetchMode(ADODB_FETCH_NUM);
  			$alltables = $db->MetaTables('TABLES');
  			$output  = '';
  			$output .= '# Backup of database \'' . $w2PgetParam['dbname'] . '\'' . "\r\n";
  			$output .= '# Generated on ' . date('j F Y, H:i:s') . "\r\n";
  			$output .= '# OS: ' . PHP_OS . "\r\n";
  			$output .= '# PHP version: ' . PHP_VERSION . "\r\n";
  			if ($w2PgetParam['dbtype'] == 'mysql')
    				$output .= '# MySQL version: ' . mysql_get_server_info() . "\r\n";
  			$output .= "\r\n";
 			$output .= "\r\n";

			// fetch all tables on by one
			foreach ($alltables as $table)
  			{
    				// introtext for this table
    				$output .= '# TABLE: ' . $table . "\r\n";
    				$output .= '# --------------------------' . "\r\n";
    				$output .= '#' . "\r\n";
    				$output .= "\r\n";
    
    				if ($drop_table) 
    				{
      					// drop table
				        $output .= 'DROP TABLE IF EXISTS `' . $table . '`;' . "\r\n";
      					$output .= "\r\n";
    				}
    
				if ($export_what != 'data') 
				{
					// structure of the table
					$rs = $db->Execute('SELECT * FROM ' . $table . ' WHERE -1');

					$fields = $db->MetaColumns($table);
					$indexes = $db->MetaIndexes($table);
					$output .= 'CREATE TABLE `' . $table . '` (' . "\r\n";
					$primary = array();
					$first = true;
					if (is_array($fields)) {
						foreach ($fields as $details) {
							if ($first)
								$first = false;
							else
								$output .= ",\r\n";
							if ($details->primary_key)
	    							$primary[] = $details->name;
	  						$output .= '  `' . $details->name . '` ' . $details->type;
	  						if ($details->max_length > -1) {
	    							$output .= '(' . $details->max_length;
	    						if (isset($details->scale))
	      							$output .= ',' . $details->scale;
	    						$output .= ')';
	  					}
	  					if ($details->not_null)
	    						$output .= ' NOT NULL';
		  				if ($details->has_default)
		    					$output .= ' DEFAULT ' . "'$details->default_value'";
		  				if ($details->auto_increment)
		    					$output .= ' auto_increment';
					}
      				}
      				if (is_array($indexes)) {
					foreach ($indexes as $index => $details) {
	  					if ($first)
	    						$first = false;
	  					else
	    						$output .= ",\r\n";
	  					$output .= '  ';
	  					if ($details['unique'])
	    						$output .= 'UNIQUE ';
	  					$output .= 'KEY `' . $index . '` ( `' . implode('`, `', $details['columns'] ) . '` )';
					}
      				}
      				if (count($primary)) {
					$output .= "\r\n" . '  PRIMARY KEY ( `'. implode('`, `', $primary) . '` )';
      				}
      				$output .= "\r\n" . ');' . "\r\n\r\n";
    			}
    
    			if ($export_what != 'table') 
    			{
      				// all data from table
      				$db->setFetchMode(ADODB_FETCH_ASSOC) ;

      				$result = $db->Execute('SELECT * FROM '.$table);
      				while($tablerow = $result->fetchRow())
      				{
        				$output .= 'INSERT INTO `'.$table.'` ( `' . implode('`, `', array_keys($tablerow)) . '` )' . "\r\n";
					$output .= ' VALUES (';
					$first = true;
					foreach ($tablerow as $value) {
	  					if ($first)
	    						$first = false;
	  					else
	    						$output .= ',';
          					// remove all enters from the field-string. MySql stamement must be on one line
          					$value = str_replace("\r\n",'\n',$value);
	  					$value = str_replace("\n", '\n', $value); // Just in case there are unadorned newlines.
          					// replace ' by \'
          					$value = str_replace('\'',"\'",$value);
          					$output .= '\''.$value.'\'';
					}
        				$output .= ');' . "\r\n";
      				} // while
      				$output .= "\r\n";
      				$output .= "\r\n";
    			}
		}
	//}
	
		switch ($output_format) {
		case 'xml':
			header('Content-Disposition: attachment; filename="backup.xml"');
			header('Content-Type: text/xml');
    			echo $output;
    		break;
  		case 'zip':
    			header('Content-Disposition: inline; filename="backup.zip"');
    			header('Content-Type: application/x-zip');
    			$zip = new zipfile;
    			$zip->addFile($output,'backup.sql');
    			echo $zip->file();
    		break;
		case 'sql':
    			header('Content-Disposition: inline; filename="backup.sql"');
    			header('Content-Type: text/sql');
    			echo $output;
    		break;
		}

		return true;
	}

	public function restore()
	{
		$upload_tmp_file = $_FILES['xmlfile']['tmp_name'];
		$continue = w2PgetParam($_POST, 'continue', false);

		$schema = new adoSchema($GLOBALS['db']);
		$schema->setUpgradeMethod('REPLACE');
		$schema->ContinueOnError(true);
		if (($sql = $schema->ParseSchemaFile($upload_tmp_file)) == false) {
  			$AppUI->setMsg('Error in parsing XML file', UI_MSG_ERR);
  			$AppUI->redirect();
		}

		$result = 0;
		$AppUI->setMsg('');
		$errs = array();
		echo '<pre>' . "\n";
		foreach ($sql as $query) {
		  if ( $db->Execute($query)) {
		    if (! $result)
		      $result = 2;
		  } else {
		    echo 'Error in Query: ' . $query . "\n";
		    echo 'Error: ' . $db->ErrorMsg() . "\n";
		    if (! $continue) {
		      $result = 0;
		      break;
		    } else {
		      $result = 1;
		    }
		  }
		}
		echo "</pre>\n";

		switch ($result) {
  		case 0:
		    echo "<B>" . $AppUI->_('Failed to restore backup') . "</b>";
		    break;
		case 1:
		    echo "<B>" . $AppUI->_('Backup restored, but with errors') . "</b>";
		    break;
		case 2:
		    echo "<B>" . $AppUI->_('Backup Restored OK') . "</b>";
		    break;
		}
		echo '<br/><b>' . $AppUI->_('xmlLoginMsg') . '</b>';
		if ($result == 0)
			return false;
		return true;
	}

?>
